@@ -1,5 +1,5 @@
 package App::ansifold;
-our $VERSION = "1.32";
+our $VERSION = "1.33";
 
 use v5.14;
 use warnings;
@@ -52,6 +52,12 @@ use Getopt::EX::Hashed 'has'; {
     has lefthalf   => '   =s  ' , default => "\N{LEFT HALF BLACK CIRCLE}";
     has righthalf  => '   =s  ' , default => "\N{RIGHT HALF BLACK CIRCLE}";
     has smart      => ' s !   ' ;
+    has crmode     => '   !   ' ;
+
+    has '+crmode' => sub {
+	$_->{$_[0]} = $_[1];
+	$_->separate = "\r" if $_[1];
+    };
     has expand     => ' x :-1 ' , default => $DEFAULT_EXPAND;
     has tabstop    => '   =i  ' , min => 1;
     has tabhead    => '   =s  ' ;
@@ -252,6 +258,7 @@ sub doit {
 	my $chomped = s/(\R+)\z// ? $1 : '';
 	my $eol = $chomped =~ /\r/ ? "\r\n" : "\n";
 	fill_paragraph() if $app->refill;
+	fill_cr() if $app->crmode;
 	if ($app->{indent_pat} && /^$app->{indent_pat}/p) {
 	    my $matched = ${^MATCH};
 	    my $indent = ansi_width $matched;
@@ -286,10 +293,33 @@ sub doit {
     return $app;
 }
 
+sub InConcatScript {
+    return <<"END";
++App::ansifold::InFullwidth
+-utf8::Hangul
+END
+}
+
+sub InFullwidthPunctuation {
+    return <<"END";
++App::ansifold::InFullwidth
+&utf8::Punctuation
+END
+}
+
+sub fill_up {
+    my $pat = shift // qr/\R/;
+    s/(?<=\p{InFullwidthPunctuation})$pat//g;
+    s/(?<=\p{InConcatScript})$pat(?=\p{InConcatScript})//g;
+    s/[ ]*$pat[ ]*/ /g;
+}
+
 sub fill_paragraph {
-    s/(?<=\p{InFullwidth})(?<=\pP)\R//g;
-    s/(?<=\p{InFullwidth})\R(?=\p{InFullwidth})//g;
-    s/[ ]*\R[ ]*/ /g;
+    fill_up(qr/\R/);
+}
+
+sub fill_cr {
+    fill_up(qr/\r/);
 }
 
 sub terminal_width {
